/*
 * CLIF is a Load Injection Framework Copyright (C) 2005 France Telecom R&D
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * CLIF
 * 
 * Contact: clif@ow2.org
 */
package org.ow2.clif.scenario.isac.engine.nodes;

import java.util.ArrayList;
import java.util.ListIterator;
import java.util.Map;
import java.util.Stack;

import org.jdom.Attribute;
import org.jdom.Element;
import org.ow2.clif.scenario.isac.engine.PlugIn;
import org.ow2.clif.scenario.isac.engine.instructions.ExitInstruction;
import org.ow2.clif.scenario.isac.engine.instructions.Instruction;
import org.ow2.clif.scenario.isac.engine.instructions.TestInstruction;

/**
 * @author Emmanuel Varoquaux
 */
public class BehaviorNode {
	public String		id;
	public ArrayList<InstructionNode>	instructions;

	public BehaviorNode(Element element) throws NodeException {
		ListIterator i;
		Attribute a;
		String name;

		for (i = element.getAttributes().listIterator(); i.hasNext();) {
			a = (Attribute)i.next();
			name = a.getName();
			if (name.equals("id")) {
				if (id != null)
					throw new DuplicatedAttributeException("id");
				id = a.getValue();
			}
			else
				throw new UnexpectedAttributeException(name);
		}
		if (id == null)
			throw new MissingAttributeException("id");

		instructions = Util.analyseBlock(element.getChildren().listIterator());
	}

	@Override
	public String toString()
	{
		StringBuilder res = new StringBuilder("behavior: id=\"" + id + "\"");
		for (int i = 0, n = instructions.size(); i < n; i++)
		{
			res.append(instructions.get(i).indentedToString(1, res));
		}
		return res.toString();
	}

	public void compile(Map<String,PlugIn> plugIns, ArrayList<Instruction> code) throws Exception
	{
		for (int i = 0, n = instructions.size(); i < n; i++)
		{
			instructions.get(i).compile(plugIns, code, new Stack<TestInstruction>());
		}
		code.add(new ExitInstruction());
	}
}
